@@ -15,7 +15,7 @@ from astropy.time import Time, TimeDelta
 from matplotlib import pyplot as plt
 from sgp4.api import Satrec
 
-from satmad.coordinates.frames import init_rvt
+from satmad.coordinates.frames import init_pvt
 from satmad.coordinates.trajectory import Trajectory
 from satmad.propagation.tle import TLE
 
@@ -50,7 +50,7 @@ def propagation_engine(line1, line2, time_list):
     ).with_differentials(vel_list)
 
     # trajectory in astropy
-    traj_astropy = init_rvt(TEME, time_list, pos_list)
+    traj_astropy = init_pvt(TEME, time_list, pos_list)
 
     # Init trajectory in Trajectory object
     trajectory = Trajectory(traj_astropy)
