@@ -15,8 +15,8 @@ from astropy.coordinates import CartesianDifferential, CartesianRepresentation
 from astropy.time import Time
 from astropy.units import Quantity
 
-from satmad.coordinates.frames import init_rvt
-from satmad.core.celestial_bodies import EARTH
+from satmad.coordinates.frames import init_pvt
+from satmad.core.celestial_bodies_lib import EARTH
 
 
 class AbstractKeplerianOrbitElements(ABC):
@@ -81,7 +81,7 @@ class AbstractKeplerianOrbitElements(ABC):
         self._self_check()
 
     def _self_check(self):
-        """Check the elements for errors. """
+        """Check the elements for errors."""
         # If conic section is too close to singular, raise error.
         if np.abs(self.sm_axis * (1 - self.eccentricity)) < 1e-3 * u.km:
             raise ValueError(
@@ -416,7 +416,7 @@ class OsculatingKeplerianOrbElems(AbstractKeplerianOrbitElements):
             init_coords_converted = init_coords[0]
 
         # Check for the initial coordinate system, convert if necessary
-        if central_body.inert_coord != init_coords_converted.frame.name:
+        if central_body.inert_coord.name != init_coords_converted.frame.name:
             init_coords_converted = init_coords_converted.transform_to(
                 central_body.inert_coord
             )
@@ -499,7 +499,7 @@ class OsculatingKeplerianOrbElems(AbstractKeplerianOrbitElements):
             unit=u.km / u.s,
         )
 
-        rv_init = init_rvt(self.central_body.inert_coord, time, r.with_differentials(v))
+        rv_init = init_pvt(self.central_body.inert_coord, time, r.with_differentials(v))
 
         return rv_init
 
@@ -672,7 +672,7 @@ def _force_angle_range(
     """
 
     while not (min_range <= angle < max_range):
-        if max_range < angle:
+        if max_range <= angle:
             angle = angle - 2 * np.pi * u.rad
 
         if angle < min_range:
