@@ -19,9 +19,9 @@ from astropy.coordinates import (
 from astropy.time import Time
 from pytest import approx
 
-from satmad.coordinates.frames import MoonCRS, init_rvt
+from satmad.coordinates.frames import MoonCRS, init_pvt
 from satmad.coordinates.tests.test_baseframe_transform import pos_err, vel_err
-from satmad.core.central_body import CelestialBody, CelestialBodyEllipsoid
+from satmad.core.celestial_body import CelestialBody, CelestialBodyEllipsoid
 from satmad.propagation.numerical_propagators import ODESolverType
 from satmad.propagation.tests.num_prop_analysis_engine import (
     energy_along_trajectory,
@@ -37,7 +37,7 @@ def rv_init_moon_crs():
     # GMAT test case
     v_moon_crs = CartesianDifferential([1, -1, 0.6], unit=u.km / u.s)
     r_moon_crs = CartesianRepresentation([1000, 1000, 2000], unit=u.km)
-    rv_init = init_rvt(MoonCRS, time, r_moon_crs.with_differentials(v_moon_crs))
+    rv_init = init_pvt(MoonCRS, time, r_moon_crs.with_differentials(v_moon_crs))
 
     return rv_init
 
@@ -51,7 +51,7 @@ _MOONGMAT = CelestialBody(
         1738.2 * u.km,
         np.inf * u.dimensionless_unscaled,
     ),
-    inert_coord="mooncrs",
+    inert_coord=MoonCRS,
 )
 """This is the Moon as defined by GMAT"""
 
@@ -98,7 +98,7 @@ def test_num_propagator_moon(rv_init_moon_crs):
         [1.870952777669765e03, -1.811234513779054e02, 2.305452195816935e03],
         unit=u.km,
     )
-    rv_fin_gmat = init_rvt(
+    rv_fin_gmat = init_pvt(
         MoonCRS, time, r_moon_crs_gmat.with_differentials(v_moon_crs_gmat)
     )
 
@@ -125,7 +125,7 @@ def rv_init_leo_gcrs():
         [5102.50895290, 6123.01139910, 6378.13693380], unit=u.km
     )
 
-    rv_init = init_rvt(GCRS, time, r_gcrs.with_differentials(v_gcrs))
+    rv_init = init_pvt(GCRS, time, r_gcrs.with_differentials(v_gcrs))
     return rv_init
 
 
