@@ -9,11 +9,11 @@ Time interval module.
 `TimeInterval` class stores time intervals and `TimeIntervalList` class stores
 lists of `TimeInterval` objects.
 """
-
 import portion as p
 from astropy import units as u
 from astropy.time import Time, TimeDelta
 from astropy.units import Quantity
+from portion import Interval
 
 _EPS_TIME = 10 * u.ns
 """Allowable time threshold, this much 'out of bounds' is allowed when assuming two
@@ -28,51 +28,60 @@ class TimeInterval:
     This is a thin wrapper around the  :class:`portion.interval.Interval` class
     from `portion` package (for the Atomic intervals), using Astropy `Time` classes
     under the hood.
-
-
-    Note that:
-
-    * only the first element of `start_time` and `end_time` objects are used,
-      even if they are defined as arrays.
-
-    * `end_time` should be greater than (or later than) `start_time`.
-      Zero duration time intervals are not allowed and will raise `ValueError`.
-
-    Parameters
-    ----------
-    start_time : Time or TimeInterval
-        Initial time to mark the start of the interval (only the first
-        instance contained in the `Time` object is used). If specified as TimeInterval,
-        it is copied into this and `end_time` is ignored.
-    end_time : Time, TimeDelta, Quantity or None
-        End time or duration to mark the end of the interval
-        (only the first instance contained in the `Time` or `TimeDelta` object is used)
-        None is acceptable only when `start_time` is defined as a `TimeInterval`
-    replicate : bool
-        `True` to replicate (deep copy) the `Time` or `TimeDelta` objects, `False` to
-        use a shallow copy to save memory
-    start_inclusive : bool
-        True if the start of the interval is inclusive (closed), False if exclusive
-        (open)
-    end_inclusive : bool
-        True if the start of the interval is inclusive (closed), False if exclusive
-        (open)
-
-    Raises
-    ------
-    ValueError
-        Raised if `end_time` is not an instance of `Time` or `TimeDelta`. Also raised
-        if `end_time` <= `start_time`.
     """
 
-    def __init__(
-        self,
+    _interval: Interval = None
+    _as_time = None
+
+    def __new__(
+        cls,
         start_time,
         end_time,
         replicate=False,
         start_inclusive=True,
         end_inclusive=True,
     ):
+        """
+        Initialises the `TimeInterval` object.
+
+        Note that:
+
+        * only the first element of `start_time` and `end_time` objects are used,
+          even if they are defined as arrays.
+
+        * `end_time` should be greater than (or later than) `start_time`.
+          Zero duration time intervals are not allowed and will raise `ValueError`.
+
+        Upon successful initialisation it will return `TimeInterval`. If an empty
+        interval is defined, it will return `None`.
+
+        Parameters
+        ----------
+        start_time : Time or TimeInterval
+            Initial time to mark the start of the interval (only the first
+            instance contained in the `Time` object is used). If specified as
+            TimeInterval, it is copied into this and `end_time` is ignored.
+        end_time : Time, TimeDelta, Quantity or None
+            End time or duration to mark the end of the interval
+            (only the first instance contained in the `Time` or `TimeDelta` object
+            is used). None is acceptable only when `start_time` is defined as a
+            `TimeInterval`
+        replicate : bool
+            `True` to replicate (deep copy) the `Time` or `TimeDelta` objects, `False`
+            to use a shallow copy to save memory
+        start_inclusive : bool
+            True if the start of the interval is inclusive (closed), False if exclusive
+            (open)
+        end_inclusive : bool
+            True if the start of the interval is inclusive (closed), False if exclusive
+            (open)
+
+        Raises
+        ------
+        ValueError
+            Raised if `end_time` is not an instance of `Time` or `TimeDelta`. Also
+            raised if `end_time` <= `start_time`.
+        """
 
         start_interval = None
         end_interval = None
@@ -81,12 +90,8 @@ class TimeInterval:
             start_inclusive = start_time.p_interval.left
             end_inclusive = start_time.p_interval.right
 
-            if replicate:
-                start_interval = start_time.start.replicate()
-                end_interval = start_time.end.replicate()
-            else:
-                start_interval = start_time.start
-                end_interval = start_time.end
+            start_interval = start_time.start
+            end_interval = start_time.end
         else:
             # Check whether init and end times are scalar or array
             if start_time.isscalar:
@@ -103,12 +108,8 @@ class TimeInterval:
             if isinstance(end_tmp, TimeDelta):
                 # end time is a TimeDelta (duration)
 
-                if replicate:
-                    start_interval = start_tmp.replicate()
-                    end_interval = start_tmp + end_tmp
-                else:
-                    start_interval = start_tmp
-                    end_interval = start_tmp + end_tmp
+                start_interval = start_tmp
+                end_interval = start_tmp + end_tmp
 
             elif isinstance(end_tmp, Quantity):
                 # end time is a Quantity, but check whether it is a valid Time Quantity
@@ -119,16 +120,16 @@ class TimeInterval:
             elif isinstance(end_tmp, Time) and not isinstance(end_tmp, TimeDelta):
                 # end time is a Time
 
+                # check for empty instances
+                if _are_times_almost_equal(start_tmp, end_tmp):
+                    return None
+
                 # replicate or shallow copy start and end values - use the first
                 # time instances only
                 if start_tmp < end_tmp:
                     # end time is later than start time - no probs
-                    if replicate:
-                        start_interval = start_tmp.replicate()
-                        end_interval = end_tmp.replicate()
-                    else:
-                        start_interval = start_tmp
-                        end_interval = end_tmp
+                    start_interval = start_tmp
+                    end_interval = end_tmp
                 else:
                     # end time is earlier than start time - raise error
                     raise ValueError(
@@ -141,15 +142,24 @@ class TimeInterval:
                     f"only Time, Quantity (Temporal) or TimeDelta classes are allowed."
                 )
 
-        # Initialise the interval
-        self._interval = p.closed(start_interval, end_interval).replace(
-            left=start_inclusive, right=end_inclusive
-        )
+        # check for empty instances
+        if _are_times_almost_equal(start_interval, end_interval):
+            return None
 
-        # Raise exception if the interval duration is too short
-        # Duration guaranteed to be positive thanks to the juggling above
-        if self.duration <= _EPS_TIME:
-            raise ValueError("Duration of the interval is negative or zero.")
+        # Initialise the interval
+        if replicate:
+            _interval = p.closed(
+                start_interval.replicate(), end_interval.replicate()
+            ).replace(left=start_inclusive, right=end_inclusive)
+        else:
+            _interval = p.closed(start_interval, end_interval).replace(
+                left=start_inclusive, right=end_inclusive
+            )
+
+        obj = super().__new__(cls)
+        obj._interval = _interval
+        obj._as_time = Time([obj.start, obj.end])
+        return obj
 
     def is_in_interval(self, time):
         """
@@ -166,14 +176,18 @@ class TimeInterval:
             True if time is within the reference interval, False otherwise
         """
         # check upper and lower boundaries for tolerance
-        if _are_times_almost_equal(self.start, time):
+        [is_start_equal, is_end_equal] = _are_times_almost_equal(self.as_time, time)
+
+        # check start
+        if is_start_equal:
             # time at starting edge - is edge closed?
             if self._interval.left:
                 return True
             else:
                 return False
 
-        if _are_times_almost_equal(self.end, time):
+        # check end
+        if is_end_equal:
             # time at end edge - is edge closed?
             if self._interval.right:
                 return True
@@ -204,10 +218,7 @@ class TimeInterval:
         start_equal = _are_times_almost_equal(interval.start, self.start)
         end_equal = _are_times_almost_equal(interval.end, self.end)
 
-        if start_equal and end_equal:
-            return True
-        else:
-            return False
+        return True if start_equal and end_equal else False
 
     def contains(self, interval):
         """
@@ -376,6 +387,11 @@ class TimeInterval:
                 "Duration of the expanded/shrunk interval is negative or zero."
             )
 
+    @property
+    def as_time(self):
+        """Gets the start and end intervals contained a single Time object."""
+        return self._as_time
+
     @property
     def duration(self):
         """
@@ -440,10 +456,11 @@ def _are_times_almost_equal(t1, t2):
 
     Returns
     -------
-    True if times are almost equal, False otherwise
+    bool
+        True if times are almost equal, False otherwise
 
     """
-    return abs(t1 - t2) < _EPS_TIME
+    return abs((t1 - t2).to_value(u.s)) < _EPS_TIME.to_value(u.s)
 
 
 def _create_interval_from_portion(interval, replicate=False):
@@ -486,7 +503,7 @@ class TimeIntervalList:
 
     Parameters
     ----------
-    intervals : list[TimeInterval]
+    intervals : list[TimeInterval] or None
         List of intervals
     start_valid : Time or TimeInterval
         Time at which the validity of this `TimeInterval` starts (only the first
@@ -640,10 +657,7 @@ class TimeIntervalList:
             if interval_member.is_intersecting(interval)
         ]
 
-        if len(intersect_intervals) > 0:
-            return True
-        else:
-            return False
+        return True if len(intersect_intervals) > 0 else False
 
     def intersect(self, interval):
         """
@@ -700,7 +714,8 @@ class TimeIntervalList:
         -------
         TimeIntervalList
            A new interval list that is the Intersection of `interval_list` and
-           this time interval list, or None if there is no intersection.
+           this time interval list, or None if there is no intersection even in the
+           validity intervals.
 
         """
         if not self.valid_interval.is_intersecting(interval_list.valid_interval):
@@ -711,6 +726,11 @@ class TimeIntervalList:
             interval_list.valid_interval
         )
 
+        if not interval_list.intervals or not self.intervals:
+            # There are no intervals to intersect in either self or the target
+            # intervals, hence no intersection possible
+            return TimeIntervalList(None, start_valid=common_valid_interval)
+
         # Compute the portion intervals
         p_self_intervals = TimeIntervalList._to_p_intervals(self.intervals)
         p_other_intervals = TimeIntervalList._to_p_intervals(interval_list.intervals)
@@ -899,9 +919,13 @@ class TimeIntervalList:
 
         # Fill the atomic `TimeInterval` objects using the merged list
         for p_interval in p_intervals:
-            intervals.append(
-                _create_interval_from_portion(p_interval, replicate=replicate)
-            )
+
+            # check for empty instances
+            if not _are_times_almost_equal(p_interval.lower, p_interval.upper):
+                # duration not empty, add the interval
+                intervals.append(
+                    _create_interval_from_portion(p_interval, replicate=replicate)
+                )
 
         return intervals
 
@@ -926,7 +950,9 @@ class TimeIntervalList:
         p_intervals = p.empty()
         if isinstance(intervals, list):
             for interval in intervals:
-                p_intervals = p_intervals.union(interval.p_interval)
+                # make sure interval is not None
+                if interval:
+                    p_intervals = p_intervals.union(interval.p_interval)
         else:
             # intervals object is a single TimeInterval
             p_intervals = intervals.p_interval
